==> gRc_0.1.0.tar.gz: gRc/R/rcox-ScoreMatching4.R <==

## iR <- getSlot(m, "intRep")
## vccTerms <- iR$vcc
## eccTerms <- iR$ecc
## allTerms <- c(vccTerms, eccTerms)

  #ctrl      <- control
  ##ctrl$vcov <- NULL ## We don't want vcov calculated until at the end...
  #m$control <- ctrl

rcorScoreMatch <- function(m, control=m$control, trace=0){
  if (trace>=2)
    cat("..Fitting with score matching\n")
  #theta     <- rcorScoreTheta(m)
  theta        <- rconScoreTheta(m)
  vn           <- unlist(lapply(getcc(m),names))
  names(theta) <- vn

  S        <- getSlot(m, "dataRep")$S
  n        <- getSlot(m, "dataRep")$n
  
  oclass       <- class(m)
  class(m)     <- c("rcon","RCOX")
  K            <- theta2K(m, theta, scale='original')
  ## print("As RCON"); print(K)

  ## Ensure diagonals are positive
  if (min(diag(K))<0){  
    vccI <- intRep(m)$vccI
    dS <- diag(S)
    for (i in 1:length(vccI)){
      cc <- as.numeric(vccI[[i]])
      dS[cc] <- mean(dS[cc])
    }
    diag(K) <- 1/dS
  }
  ##print("After redoing diagonals");  print(K)

  if (min(eigen(K)$values)<0)  
    K     <- regularizeK(K)
  ##print("After regularizing diagonals"); print(K)

  dimnames(K)  <- dimnames(S)
  ##print(ellK(K,S,n-1))
  
  class(m) <- oclass
  K <- findKinModel(m,K,type="rcor")

  #print(K)  
  #print(ellK(K,S,n-1))
  #print(1/sqrt(diag(S)))
  
  ## K            <- theta2K(m, theta, scale="original")
  ## dimnames(K)  <- dimnames(S)

  logL   <- ellK(K, S, n-1)
  if (trace>=3)
    cat("..Score matching, logL:", logL, "\n")

  return(list(K=K, logL=logL, coef=theta))
}

rconScoreMatch <- function(m, control=m$control, trace=0){

  tstart <- proc.time()
  if (trace>=2)
    cat("..Fitting with score matching\n")

  theta <- rconScoreTheta(m)
  vn           <- unlist(lapply(getcc(m),names))
  names(theta) <- vn

  S        <- getSlot(m, "dataRep")$S
  n        <- getSlot(m, "dataRep")$n
    
  K            <- theta2K(m, theta, scale='original')
  dimnames(K)  <- dimnames(S)

  ##print("After score matching"); print(ellK(K,S,n-1))
  ## Ensure diagonals are positive
  if (min(diag(K))<0){
    vccI <- intRep(m)$vccI
    dS <- diag(S)
    for (i in 1:length(vccI)){
      cc <- as.numeric(vccI[[i]])
      dS[cc] <- mean(dS[cc])
    }
    diag(K) <- 1/dS
  }

  if (min(eigen(K)$values)<0)
    K     <- regularizeK(K)
  
  logL  <- ellK(K,S,n-1)
  ##print("After redoing diagonals"); print(logL)

  if (trace>=3)
    cat("..Score matching, logL:", logL, "\n")

  ans <- list(K=K, logL=logL, coef=theta)
  return(ans)
}


rconScoreTheta <- function(m){

  iR <- getSlot(m, "intRep")
  vccTerms <- iR$vcc
  eccTerms <- iR$ecc
  allTerms <- c(vccTerms, eccTerms)

  S    <- getSlot(m, "dataRep")$S
  n    <- getSlot(m, "dataRep")$n

  A <- matrix(0, ncol=length(allTerms), nrow=length(allTerms))
  B <- rep(0,length(allTerms))

  #print(A)
  
  for (u in 1:length(allTerms)){
    Ku <- allTerms[[u]]
    #print(Ku)
    bu     <- trA(Ku)
    B[u]   <- bu
    for (v in u:length(allTerms)){
      Kv <- allTerms[[v]]
      auv    <- trAWB(Ku, S, Kv)      
      A[v,u] <- A[u,v] <- auv
    }
  }
  #print(A); print(B)
  theta <- solve(A,B)  
  ##theta <- colSums(cholSolve(A)*B) ????  
  return(theta)
}

rcorScoreTheta <- function(m){


  theta        <- rconScoreTheta(m)
  vn           <- unlist(lapply(getcc(m),names))
  names(theta) <- vn

  S        <- getSlot(m, "dataRep")$S
  n        <- getSlot(m, "dataRep")$n

  oclass <- class(m)
  class(m) <- c("rcon","RCOX")
  K            <- theta2K(m, theta, scale='original')
  print(K)
  dimnames(K)  <- dimnames(S)
  ##print(ellK(K,S,n-1))

  ## Ensure diagonals are positive
  vccI <- intRep(m)$vccI
  dS <- diag(S)
  for (i in 1:length(vccI)){
    cc <- as.numeric(vccI[[i]])
    dS[cc] <- mean(dS[cc])
  }
  diag(K) <- 1/dS
  
  K     <- regularizeK(K) 
  logL  <- ellK(K,S,n-1)
  ##print(ellK(K,S,n-1))
  
  class(m) <- oclass
  theta    <- K2theta(m, K, scale="original")

  vn       <- unlist(lapply(getcc(m),names))
  names(theta) <- vn
  return(theta)
}


  

#   ctrl   <- m$control
#   ctrl$vcov <- NULL
#   class(m) <- c("rcon","RCOX")
#   KS       <- rconScoreMatch(m, control=ctrl)$K
#   class(m) <- oclass
#   K.curr   <- findKinModel(m, KS=KS, type='rcor', regularize=TRUE)

#   theta    <- K2theta(m, K.curr, scale="original")
#   vn       <- unlist(lapply(getcc(m),names))
#   names(theta) <- vn
#   return(theta)

#   if (!is.null(control$vcov)){
#     V <- calculateVCOV(m, K=K, vcov=control$vcov, nboot=control$nboot)
#   }




#   print(K)  
#   K <<- K
#   S <<- S
#   print(S)
#   K <<- K; S<<-S

#  print(ellK(K,S,n-1))
  
#  if (min(eigen(K)$values)<0){
#    cat("Score matching gave n.d. K; regularizing to make K p.d.\n")
#    K <- regularizeK(K)
#  }

==> gRc_0.1.2.tar.gz: gRc/R/rcox-ScoreMatching4.R <==

## iR <- getSlot(m, "intRep")
## vccTerms <- iR$vcc
## eccTerms <- iR$ecc
## allTerms <- c(vccTerms, eccTerms)

  #ctrl      <- control
  ##ctrl$vcov <- NULL ## We don't want vcov calculated until at the end...
  #m$control <- ctrl

rcorScoreMatch <- function(m, control=m$control, trace=0){
  tstart <- proc.time()
  if (trace>=2)
    cat("..Fitting with score matching\n")
  #theta     <- rcorScoreTheta(m)
  theta        <- rconScoreTheta(m)
  vn           <- unlist(lapply(getcc(m),names))
  names(theta) <- vn

  S        <- getSlot(m, "dataRep")$S
  n        <- getSlot(m, "dataRep")$n
  
  oclass       <- class(m)
  class(m)     <- c("rcon","rcox")
  K            <- theta2K(m, theta, scale='original')
  ## print("As RCON"); print(K)

  ## Ensure diagonals are positive
  if (min(diag(K))<0){  
    vccI <- intRep(m)$vccI
    dS <- diag(S)
    for (i in 1:length(vccI)){
      cc <- as.numeric(vccI[[i]])
      dS[cc] <- mean(dS[cc])
    }
    diag(K) <- 1/dS
  }
  ##print("After redoing diagonals");  print(K)

  if (min(eigen(K)$values)<0)  
    K     <- regularizeK(K)
  ##print("After regularizing diagonals"); print(K)

  dimnames(K)  <- dimnames(S)
  ##print(ellK(K,S,n-1))
  
  class(m) <- oclass
  K <- findKinModel(m,K,type="rcor")

  #print(K)  
  #print(ellK(K,S,n-1))
  #print(1/sqrt(diag(S)))
  
  ## K            <- theta2K(m, theta, scale="original")
  ## dimnames(K)  <- dimnames(S)

  logL   <- ellK(K, S, n-1)
  if (trace>=3)
    cat("..Score matching, logL:", logL, "Time:", proc.time()-tstart, "\n")


  return(list(K=K, logL=logL, coef=theta))
}

rconScoreMatch <- function(m, control=m$control, trace=0){

  tstart <- proc.time()
  if (trace>=2)
    cat("..Fitting with score matching\n")

  theta <- rconScoreTheta(m)

  ##cat("---", proc.time()-tstart,"\n")
  
  vn           <- unlist(lapply(getcc(m),names))
  names(theta) <- vn

  S        <- getSlot(m, "dataRep")$S
  n        <- getSlot(m, "dataRep")$n
    
  K            <- theta2K(m, theta, scale='original')
  dimnames(K)  <- dimnames(S)

  ## print("After score matching"); print(ellK(K,S,n-1))
  ## Ensure diagonals are positive
  if (min(diag(K))<0){
    vccI <- intRep(m)$vccI
    dS <- diag(S)
    for (i in 1:length(vccI)){
      cc <- as.numeric(vccI[[i]])
      dS[cc] <- mean(dS[cc])
    }
    diag(K) <- 1/dS
  }

  ## cat("---", proc.time()-tstart,"\n")
  
  if (min(eigen(K)$values)<0)
    K     <- regularizeK(K)
  
  logL  <- ellK(K,S,n-1)
  ##print("After redoing diagonals"); print(logL)

  if (trace>=3)
    cat("..Score matching, logL:", logL, "Time:", proc.time()-tstart, "\n")

  ans <- list(K=K, logL=logL, coef=theta)
  return(ans)
}

# rconScoreThetaDiag <- function(m){
#   iR <- getSlot(m, "intRep")
#   vccTerms <- iR$vcc
#   S    <- getSlot(m, "dataRep")$S
#   n    <- getSlot(m, "dataRep")$n

#   a <- B <- rep(0,length(vccTerms))
#    for (u in 1:length(vccTerms)){
#      Ku <- vccTerms[[u]]
#      bu     <- trA(Ku)
#      B[u]   <- bu
#      a[u]   <- trAWB(Ku, S, Ku)
#    }
#   theta <- B/a

#   theta <- c(theta, rep(0,length(iR$ecc)))
  
#   K            <- theta2K(m, theta, scale='original')
#   dimnames(K)  <- dimnames(S)
#   return(K)

# }

rconScoreTheta <- function(m){

  iR       <- getSlot(m, "intRep")
  vccTerms <- iR$vcc
  eccTerms <- iR$ecc

  lvcc <- length(vccTerms)
  lecc <- length(eccTerms)

  allTerms <- c(vccTerms, eccTerms)

  S    <- getSlot(m, "dataRep")$S
  n    <- getSlot(m, "dataRep")$n

  A <- matrix(0, ncol=lvcc+lecc, nrow=lvcc+lecc)
  B <- rep(0,length(allTerms))

  #print(A)

  for (u in 1:lvcc){
    Ku     <- allTerms[[u]]
    bu     <- trA(Ku)
    B[u]   <- bu
    A[u,u] <- trAWB(Ku, S, Ku)
  }

  if (lecc>0){
    for (u in 1:lvcc){
      Ku      <- vccTerms[[u]]
      for (v in 1:lecc){
        Kv     <- eccTerms[[v]]
        #if (!zeroAB(A,B)){
          auv         <- trAWB(Ku, S, Kv)
          A[u,v+lvcc] <- A[v+lvcc,u] <- auv
        #}
      }
    }
    
    for (u in 1:lecc){
      Ku      <- eccTerms[[u]]
      for (v in u:lecc){
        Kv     <- eccTerms[[v]]
        auv    <- trAWB(Ku, S, Kv)      
        A[u+lvcc,v+lvcc] <- A[v+lvcc,u+lvcc] <- auv      
      }
    }
  }

  #print(round(A))
#   for (u in 1:length(allTerms)){
#     Ku      <- allTerms[[u]]
#     #bu     <- trA(Ku)
#     #B[u]   <- bu
#     for (v in u:length(allTerms)){
#       Kv     <- allTerms[[v]]
#       auv    <- trAWB(Ku, S, Kv)      
#       A[v,u] <- A[u,v] <- auv
#     }
#   }

  
  ##print(dim(A))
  theta <- solve(A,B)  
  ##theta <- colSums(cholSolve(A)*B) ????  
  return(theta)
}

rcorScoreTheta <- function(m){


  theta        <- rconScoreTheta(m)
  vn           <- unlist(lapply(getcc(m),names))
  names(theta) <- vn

  S        <- getSlot(m, "dataRep")$S
  n        <- getSlot(m, "dataRep")$n

  oclass <- class(m)
  class(m) <- c("rcon","rcox")
  K            <- theta2K(m, theta, scale='original')
  print(K)
  dimnames(K)  <- dimnames(S)
  ##print(ellK(K,S,n-1))

  ## Ensure diagonals are positive
  vccI <- intRep(m)$vccI
  dS <- diag(S)
  for (i in 1:length(vccI)){
    cc <- as.numeric(vccI[[i]])
    dS[cc] <- mean(dS[cc])
  }
  diag(K) <- 1/dS
  
  K     <- regularizeK(K) 
  logL  <- ellK(K,S,n-1)
  ##print(ellK(K,S,n-1))
  
  class(m) <- oclass
  theta    <- K2theta(m, K, scale="original")

  vn       <- unlist(lapply(getcc(m),names))
  names(theta) <- vn
  return(theta)
}


  

#   ctrl   <- m$control
#   ctrl$vcov <- NULL
#   class(m) <- c("rcon","rcox")
#   KS       <- rconScoreMatch(m, control=ctrl)$K
#   class(m) <- oclass
#   K.curr   <- findKinModel(m, KS=KS, type='rcor', regularize=TRUE)

#   theta    <- K2theta(m, K.curr, scale="original")
#   vn       <- unlist(lapply(getcc(m),names))
#   names(theta) <- vn
#   return(theta)

#   if (!is.null(control$vcov)){
#     V <- calculateVCOV(m, K=K, vcov=control$vcov, nboot=control$nboot)
#   }




#   print(K)  
#   K <<- K
#   S <<- S
#   print(S)
#   K <<- K; S<<-S

#  print(ellK(K,S,n-1))
  
#  if (min(eigen(K)$values)<0){
#    cat("Score matching gave n.d. K; regularizing to make K p.d.\n")
#    K <- regularizeK(K)
#  }
